/*!
 * react-colrow v2.0.2
 * (c) phphe <phphe@outlook.com> (https://github.com/phphe)
 * Released under the MIT License.
 */
'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var hp = require('helper-js');
var React = require('react');
var React__default = _interopDefault(React);

function styleInject(css, ref) {
  if (ref === void 0) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') {
    return;
  }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = ".cr-row{\n  width: 100%;\n  overflow: hidden;\n}\n.cr-row-inner{\n  display: flex;\n  flex-wrap: wrap;\n}";
styleInject(css_248z);

// detect if need reduce col width
// detect browsers, from: https://stackoverflow.com/questions/9847580/how-to-detect-safari-chrome-ie-firefox-and-opera-browser
function isNode() {
  return typeof module !== 'undefined' && module.exports;
}

function isFirefox() {
  return typeof InstallTrigger !== 'undefined';
} // Safari 3.0+ "[object HTMLElementConstructor]" 


function isSafari() {
  return /constructor/i.test(window.HTMLElement) || function (p) {
    return p.toString() === "[object SafariRemoteNotification]";
  }(!window['safari'] || typeof safari !== 'undefined' && safari.pushNotification); // eslint-disable-line no-undef
} // Chrome 1 - 79


function isChrome() {
  return !!window.chrome && (!!window.chrome.webstore || !!window.chrome.runtime);
} // detect by DOM
// const div = document.createElement('div')
// document.body.appendChild(div)
// div.style.width='calc(1000px / 7)'
// const w = hp.getBoundingClientRect(div).width
// if (w * 7 > 1000) {
//   _ifNeedReduceColWidth = true
// }
// hp.removeEl(div)


var detectIfNeedReduceColWidth = (() => Boolean(!isNode() && !isChrome() && !isSafari() && !isFirefox()));

const RowContext = React.createContext({});

const ifNeedReduceColWidth = detectIfNeedReduceColWidth();
const config = {
  DEFAULT_GUTTER_X: 16,
  DEFAULT_GUTTER_Y: 16,
  COL_WIDTH_REDUCE: 0.09,
  BREAK_POINTS: {
    xs: 544,
    sm: 768,
    md: 992,
    lg: 1200
  },
  ROW_HEIGHT_CALCULATION: true
};
const defaultProps = {
  gutter: [config.DEFAULT_GUTTER_X, config.DEFAULT_GUTTER_Y],
  heightCalculation: config.ROW_HEIGHT_CALCULATION,
  breakPoints: config.BREAK_POINTS
};
function Row(props = {}) {
  props = Object.assign(Object.assign({}, defaultProps), props);
  const className = `cr-col-${hp.strRand(6)}`;
  const {
    gutter,
    breakPoints,
    xsGutterX,
    xsGutterY,
    smGutterX,
    smGutterY,
    mdGutterX,
    mdGutterY,
    lgGutterX,
    lgGutterY,
    xlGutterX,
    xlGutterY
  } = props; // state

  const resolveGutterXY = () => {
    const t = Array.isArray(gutter) ? gutter.slice() : [gutter, gutter];

    if (t[0] == null) {
      t[0] = config.DEFAULT_GUTTER_X;
    }

    if (t[1] == null) {
      t[1] = config.DEFAULT_GUTTER_Y;
    }

    return t;
  };

  const initialGutter = resolveGutterXY();
  const [gutterX, setgutterX] = React.useState(initialGutter[0]);
  const [gutterY, setgutterY] = React.useState(initialGutter[1]);
  const [innerHeight, setinnerHeight] = React.useState(); // ref

  const inner = React.useRef(null);

  const updateInnerHeight = () => {
    if (inner.current != null) {
      const el = inner.current;
      const h = hp.getBoundingClientRect(el).height;

      if (h !== innerHeight) {
        setinnerHeight(h);
      }
    }
  }; // computed


  const styleText = React.useMemo(() => {
    const baseStyleText = (argGutterX, argGutterY) => {
      if (argGutterX == null) {
        argGutterX = gutterX;
      }

      if (argGutterY == null) {
        argGutterY = gutterY;
      }

      let styleText = `.${className}{\n`;
      styleText += `margin-right: -${argGutterX}px;`;

      if (innerHeight == null) {
        styleText += `margin-bottom: -${argGutterY}px;`;
      } else if (innerHeight !== 0) {
        styleText += `height: ${innerHeight - argGutterY}px;`;
      }

      styleText += `}`;
      styleText += `.${className} > .cr-row-inner{
        width: calc(100% + ${argGutterX}px);
      }`;
      return styleText;
    };

    let styleText = baseStyleText(gutterX, gutterY); // responsive

    const bp = breakPoints;

    if (xsGutterX != null || xsGutterY != null) {
      styleText += `@media (max-width: ${bp.xs}px) {${baseStyleText(xsGutterX, xsGutterY)}}`;
    }

    if (smGutterX != null || smGutterY != null) {
      styleText += `@media (min-width: ${bp.xs}px) {${baseStyleText(smGutterX, smGutterY)}}`;
    }

    if (mdGutterX != null || mdGutterY != null) {
      styleText += `@media (min-width: ${bp.sm}px) {${baseStyleText(mdGutterX, mdGutterY)}}`;
    }

    if (lgGutterX != null || lgGutterY != null) {
      styleText += `@media (min-width: ${bp.md}px) {${baseStyleText(lgGutterX, lgGutterY)}}`;
    }

    if (xlGutterX != null || xlGutterY != null) {
      styleText += `@media (min-width: ${bp.lg}px) {${baseStyleText(xlGutterX, xlGutterY)}}`;
    } // 


    return `<style type="text/css">${styleText}</style>`.replace(/\n/g, '');
  }, [gutterX, gutterY, breakPoints, xsGutterX, xsGutterY, smGutterX, smGutterY, mdGutterX, mdGutterY, lgGutterX, lgGutterY, xlGutterX, xlGutterY, className, innerHeight]); // watch

  const updateGutterXY = () => {
    const t = resolveGutterXY();
    setgutterX(t[0]);
    setgutterY(t[1]);
  };

  React.useEffect(updateGutterXY, [gutter]); // mounted

  React.useEffect(() => {
    if (props.heightCalculation) {
      updateInnerHeight();
      hp.onDOM(window, 'resize', updateInnerHeight);
      let observer;

      if (window.MutationObserver) {
        // Select the node that will be observed for mutations
        const targetNode = document.body.parentNode;

        if (targetNode) {
          // Options for the observer (which mutations to observe)
          const config = {
            attributes: true,
            childList: true,
            subtree: true
          }; // Callback function to execute when mutations are observed

          const callback = () => {
            updateInnerHeight();
          }; // Create an observer instance linked to the callback function


          observer = new MutationObserver(callback); // Start observing the target node for configured mutations

          observer.observe(targetNode, config);
        }
      } // destroy


      return () => {
        hp.offDOM(window, 'resize', updateInnerHeight);

        if (observer) {
          // Later, you can stop observing
          observer.disconnect();
        }
      };
    }
  }, []); // eslint-disable-line react-hooks/exhaustive-deps
  // render

  return React__default.createElement(RowContext.Provider, {
    value: {
      gutterX,
      gutterY,
      breakPoints,
      xsGutterX,
      xsGutterY,
      smGutterX,
      smGutterY,
      mdGutterX,
      mdGutterY,
      lgGutterX,
      lgGutterY,
      xlGutterX,
      xlGutterY
    }
  }, React__default.createElement("div", {
    className: `cr-row ${props.className || ''} ${className}`,
    style: props.style
  }, React__default.createElement("div", {
    className: "cr-row-inner",
    ref: inner
  }, props.children), React__default.createElement("div", {
    className: "cr-dynamic-style",
    style: {
      display: 'none'
    },
    dangerouslySetInnerHTML: {
      __html: styleText
    }
  })));
}

var css_248z$1 = ".cr-col{\n  box-sizing: border-box;\n}";
styleInject(css_248z$1);

const defaultProps$1 = {
  colWidthReduce: config.COL_WIDTH_REDUCE
};
function Col(props = {}) {
  props = Object.assign(Object.assign({}, defaultProps$1), props);
  const className = `cr-col-${hp.strRand(6)}`;
  const {
    width,
    grow,
    xs,
    xsGrow,
    sm,
    smGrow,
    md,
    mdGrow,
    lg,
    lgGrow,
    xl,
    xlGrow,
    colWidthReduce
  } = props;
  const ctx = React.useContext(RowContext);
  const {
    gutterX,
    gutterY,
    breakPoints,
    xsGutterX,
    xsGutterY,
    smGutterX,
    smGutterY,
    mdGutterX,
    mdGutterY,
    lgGutterX,
    lgGutterY,
    xlGutterX,
    xlGutterY
  } = ctx; // computed 

  const styleText = React.useMemo(() => {
    const baseStyleText = (width, grow, gutterX, gutterY) => {
      // convert width to css text
      const widthText = (width, gutterX) => {
        if (width == null) {
          width = props.width;
        }

        if (gutterX == null) {
          gutterX = ctx.gutterX;
        }

        if (typeof width === 'number') {
          if (width <= 1) {
            const reduce = ifNeedReduceColWidth ? ` - ${colWidthReduce}px` : '';
            return `calc(100% * ${width} - ${gutterX}px${reduce})`;
          } else {
            return `${width}px`;
          }
        } else {
          return width; // such as 100px, 100em, 10cm
        }
      };

      let empty = true;
      const styles = [];

      if (gutterX != null) {
        styles.push(`margin-right: ${gutterX}px;`);
        empty = false;
      }

      if (gutterY != null) {
        styles.push(`margin-bottom: ${gutterY}px;`);
        empty = false;
      }

      if (width == null && grow) {
        width = 2;
      }

      if (width != null || gutterX != null) {
        styles.push(`width: ${widthText(width, gutterX)};`);
        empty = false;
      }

      if (grow != null && grow !== false) {
        if (grow === true) {
          grow = 1;
        }

        styles.push(`flex-grow: ${grow};`);
        empty = false;
      }

      const style = `.${className}{${styles.join('')}}`;
      return {
        empty,
        style
      };
    };

    let styleText = ``;
    let w = width;

    if (w == null && !grow) {
      w = 1;
    }

    styleText += baseStyleText(w, grow, gutterX, gutterY).style; // responsive

    const bp = breakPoints;
    let t;
    t = baseStyleText(xs, xsGrow, xsGutterX, xsGutterY);

    if (!t.empty) {
      styleText += `
        @media (max-width: ${bp.xs}px) {
          ${t.style}
        }
      `;
    }

    t = baseStyleText(sm, smGrow, smGutterX, smGutterY);

    if (!t.empty) {
      styleText += `
        @media (min-width: ${bp.xs}px) {
          ${t.style}
        }
      `;
    }

    t = baseStyleText(md, mdGrow, mdGutterX, mdGutterY);

    if (!t.empty) {
      styleText += `
        @media (min-width: ${bp.sm}px) {
          ${t.style}
        }
      `;
    }

    t = baseStyleText(lg, lgGrow, lgGutterX, lgGutterY);

    if (!t.empty) {
      styleText += `
        @media (min-width: ${bp.md}px) {
          ${t.style}
        }
      `;
    }

    t = baseStyleText(xl, xlGrow, xlGutterX, xlGutterY);

    if (!t.empty) {
      styleText += `
        @media (min-width: ${bp.lg}px) {
          ${t.style}
        }
      `;
    } // 


    return `<style type="text/css">${styleText}</style>`.replace(/\n/g, '');
  }, [width, grow, xs, xsGrow, sm, smGrow, md, mdGrow, lg, lgGrow, xl, xlGrow, colWidthReduce, gutterX, gutterY, breakPoints, xsGutterX, xsGutterY, smGutterX, smGutterY, mdGutterX, mdGutterY, lgGutterX, lgGutterY, xlGutterX, xlGutterY, className, ctx.gutterX, props.width]); // render

  return React__default.createElement("div", {
    className: `cr-col ${className} ${props.className || ''}`,
    style: props.style
  }, props.children, React__default.createElement("div", {
    className: "cr-dynamic-style",
    style: {
      display: 'none'
    },
    dangerouslySetInnerHTML: {
      __html: styleText
    }
  }));
}

var css_248z$2 = ".cr-break-row{\n  width: 100%;\n}";
styleInject(css_248z$2);

function BreakRow(props = {}) {
  const className = `cr-break-row-${hp.strRand(6)}`;
  const {
    xs,
    sm,
    md,
    lg,
    xl
  } = props;
  const {
    breakPoints
  } = React.useContext(RowContext); // computed

  const styleText = React.useMemo(() => {
    if (xs || sm || md || lg || xl) {
      let styleText = `.${className}{display: none;}`;
      const bp = breakPoints;

      if (xs) {
        styleText += `
          @media (max-width: ${bp.xs}px){
            .${className}{display: block;}
          }
        `;
      }

      if (sm) {
        styleText += `
          @media (max-width: ${bp.sm}px) and (min-width: ${bp.xs}px){
            .${className}{display: block;}
          }
        `;
      }

      if (md) {
        styleText += `
          @media (max-width: ${bp.md}px) and (min-width: ${bp.sm}px){
            .${className}{display: block;}
          }
        `;
      }

      if (lg) {
        styleText += `
          @media (max-width: ${bp.lg}px) and (min-width: ${bp.md}px){
            .${className}{display: block;}
          }
        `;
      }

      if (xl) {
        styleText += `
          @media (min-width: ${bp.lg}px){
            .${className}{display: block;}
          }
        `;
      }

      return `<style type="text/css">${styleText}</style>`.replace(/\n/g, '');
    }

    return '';
  }, [xs, sm, md, lg, xl, breakPoints, className]); // render

  return React__default.createElement("div", {
    className: `cr-break-row ${className} ${props.className || ''}`,
    style: props.style
  }, React__default.createElement("div", {
    className: "cr-dynamic-style",
    style: {
      display: 'none'
    },
    dangerouslySetInnerHTML: {
      __html: styleText
    }
  }));
}

exports.BreakRow = BreakRow;
exports.Col = Col;
exports.Row = Row;
exports.config = config;
